
/**
 * @inv getWidth() > 0
 * @inv getHeight() > 0
 */
public interface Matrix {

	/**
	 * Retrieve the width of the matrix
	 */
	int getWidth();
	
	/**
	 * Retrieve the height of the matrix
	 */
	int getHeight();
	
	/**
	 * Retrieve the element at position [row, col].
	 * @pre  0 <= row < getHeight();
	 * @pre  0 <= col < getWidth();
	 */
	double getElementAt(int row, int col);

	/**
	 * Set the value of the element at position [row, col].
	 * @pre  0 <= row < getHeight();
	 * @pre  0 <= col < getWidth();
	 * @post getElementAt(row, col) == newVal;
	 */
	void setElementAt(int row, int col, double newVal);
	
	/**
	 * Multiply this matrix by another
	 * @pre other != null
	 * @pre this.getWidth() == other.getHeight()
	 */
	Matrix multiply(Matrix other);
	
	/**
	 * Multiply this matrix by a scalar
	 */
	public Matrix multiply(double scalar);
}
