
public class MatrixFactory {

	/**
	 * Create a matrix from the specified two dimensional array. 
	 * @param data - content of the matrix
	 * @return a new Matrix object. The concrete Matrix type depends
	 * 		   on the data. 
	 * 
	 * @pre data != null
	 * @pre forall i,j 0 <= i,j < data.length, data[i].length == data[j].length
	 * @post $return != null
	 * @post if i,j "within the boundaries of the matrix"
	 * 					 $return.getElementAt(i, j) == data[i][j]
	 */
	public static Matrix newMatrix(double[][] data) {
		if (isDiagonal(data)) {
			return  new DiagonalMatrix(data);
		} else if (isSparse(data)) {
			return  new SparseMatrix(data);
		} 
		return new OrdinaryMatrix(data);
	}

	private static double THRESHOLD = 0.1;
	
	private static boolean isSparse(double[][] data) {
		int count = 0,
			height = data.length,
			width = data[0].length;
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				if (data[i][j] != 0)
					count++;
			}
		}
		return ((double)count/(height * width)) < THRESHOLD; 
	}

	private static boolean isDiagonal(double[][] data) {
		for (int i = 0; i < data.length; i++) {
			for (int j = 0; j < data[i].length; j++) {
				if (i != j  &&  data[i][j] != 0)
					return false;
			}
		}
		return true;
	}
}
