import java.util.Arrays;


public class OrdinaryMatrix implements Matrix {

	/**
	 * @pre row > 0
	 * @pre col > 0
	 * @post getHeight() == row
	 * @post getWidth() == col
	 */
	public OrdinaryMatrix(int height, int width) {
		array = new double[height][width];
		this.height = height;
		this.width = width;
	}

	
	public OrdinaryMatrix(double[][] data) {
		array = new double[data.length][];
		for (int i = 0; i < data.length; ++i) {
			array[i] = Arrays.copyOf(data[i], data[i].length);
		}
		height = array.length;
		width = data[0].length;
	}
	
	@Override
	public double getElementAt(int row, int col) {
		return array[row][col];
	}

	@Override
	public int getHeight() {
		return height;
	}

	@Override
	public int getWidth() {
		return width;
	}

	@Override
	public void setElementAt(int row, int col, double newVal) {
		array[row][col] = newVal;
	}

	/**
	 * Multiply this matrix by another
	 * @pre other != null
	 * @pre getWidth() == other.getHeight()
	 */
	@Override
	public Matrix multiply(Matrix other) {
		OrdinaryMatrix result = new OrdinaryMatrix(height, other.getWidth());

		for (int i = 0; i < height; ++i) {
			for (int j = 0; j < other.getWidth(); ++j) {
				for (int k = 0; k < other.getHeight(); ++k)
					result.array[i][j] += array[i][k] * other.getElementAt(k, j);
			}
		}

		return result;
	}
	
	/**
	 * Multiply this matrix by a scalar
	 */
	@Override
	public Matrix multiply(double scalar) {
		OrdinaryMatrix result = new OrdinaryMatrix(height, width);

		for (int i = 0; i < height; ++i) {
			for (int j = 0; j < width; ++j) {
				result.array[i][j] = array[i][j] * scalar;
			}
		}

		return result;
	}

	private double[][] array;
	int height, width;
	
}
